
# q-expansion of newform 1008.2.a.n, downloaded from the LMFDB on 25 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1008
weight = 2
poly_data = [-3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [0, 1], [-1, 0], [0, 1], [2, 0], [0, -1], [4, 0], [0, -1], [0, 0], [4, 0], [2, 0], [0, -3], [4, 0], [0, 2], [0, 2], [0, -2], [-10, 0], [4, 0], [0, -3], [14, 0], [-8, 0], [0, 0], [0, 1], [14, 0], [0, -1], [4, 0], [0, 5], [2, 0], [0, 0], [-8, 0], [0, -4], [0, 2], [16, 0], [0, -2], [-8, 0], [-10, 0], [-20, 0], [0, -6], [0, -5], [0, -5], [2, 0], [0, -7], [14, 0], [0, -6], [16, 0], [-20, 0], [-8, 0], [0, -2], [-22, 0], [0, -2], [0, 3], [-10, 0], [0, 6], [0, 1], [0, -5], [0, 5], [-20, 0], [-10, 0], [0, 6], [4, 0], [0, 3], [28, 0], [0, 10], [2, 0], [0, -2], [-20, 0], [14, 0], [0, -5], [14, 0], [0, -1], [0, -7], [16, 0], [-10, 0], [28, 0], [0, -4], [0, -4], [38, 0], [0, -2], [14, 0], [0, 6], [-10, 0], [0, 1], [26, 0], [16, 0], [0, -1], [0, -12], [-10, 0], [0, -9], [-32, 0], [0, 2], [0, -2], [40, 0], [0, 3], [4, 0], [0, 0], [0, 1], [0, -1], [16, 0], [14, 0], [4, 0], [0, 2], [0, 10], [0, -2], [28, 0], [2, 0], [0, -6], [0, 7], [0, 13], [-22, 0], [-8, 0], [38, 0], [0, 6], [-8, 0], [-8, 0], [0, 14], [-20, 0], [0, -2], [0, -8], [0, -3], [-10, 0], [-10, 0], [0, 7], [0, 7], [-8, 0], [0, 0], [26, 0], [0, 8], [4, 0], [-22, 0], [-20, 0], [0, -3], [-8, 0], [26, 0], [0, -11], [-22, 0], [0, -13], [-32, 0], [0, -9], [0, 8], [16, 0], [0, -2], [-32, 0], [0, -3], [50, 0], [0, 6], [-10, 0], [0, 5], [4, 0], [0, 11], [14, 0], [0, 15], [52, 0], [0, 2], [28, 0], [0, 9], [16, 0], [0, 13], [-46, 0], [0, 5], [0, -7], [0, 12], [16, 0], [0, 8], [0, -10], [0, 4], [-56, 0], [-10, 0], [-46, 0], [0, 7], [0, 13], [-58, 0], [0, 5], [2, 0], [40, 0], [0, -9], [4, 0], [0, -4], [-44, 0], [62, 0], [16, 0], [0, -12], [-34, 0], [0, 7], [0, -5], [0, 11], [2, 0], [-56, 0], [50, 0], [0, 4], [14, 0], [0, -15], [-20, 0], [0, -5], [0, 1], [0, 11], [-10, 0], [-10, 0], [0, -15], [0, -10], [0, 2], [-32, 0], [-22, 0], [-10, 0], [0, 12], [0, 11], [16, 0], [0, -19], [0, 12], [4, 0], [2, 0], [0, 9], [40, 0], [0, 8], [0, -14], [26, 0], [-8, 0], [0, -1], [0, -1], [0, 12], [-22, 0], [-20, 0], [0, 16], [40, 0], [0, 12], [50, 0], [0, -11], [0, 7], [52, 0], [0, -1], [-46, 0], [4, 0], [-56, 0], [0, 8], [4, 0], [0, 16], [62, 0], [0, -14], [0, 3], [0, 0], [0, 7], [-32, 0], [64, 0], [2, 0], [0, 9], [0, -22], [28, 0], [0, -2], [28, 0], [0, 15], [2, 0], [0, 19], [0, 13], [14, 0], [0, -13], [0, -7], [-34, 0], [-44, 0], [0, 3], [-22, 0], [16, 0], [0, -9], [50, 0], [-10, 0], [0, -13], [-8, 0], [0, -6], [0, 15], [4, 0], [0, -4], [-10, 0], [52, 0], [62, 0], [40, 0], [14, 0], [-20, 0], [0, 5], [-46, 0], [38, 0], [0, 14], [0, 4], [40, 0], [0, -12], [-58, 0], [-32, 0], [0, -7], [-34, 0], [0, -18], [28, 0], [0, -3], [0, 2], [0, -20], [0, -2], [0, -6], [14, 0], [0, -1], [64, 0], [0, -3], [0, 14], [40, 0], [-22, 0], [0, -8], [-32, 0], [0, -3], [-68, 0], [62, 0], [0, 19], [-22, 0], [0, -7], [-22, 0], [0, 20], [0, 8], [0, -14], [-68, 0], [0, 15], [-34, 0], [0, -12], [0, 19], [86, 0], [0, -6], [52, 0], [62, 0], [0, -21], [-8, 0], [0, 8], [38, 0], [-20, 0], [28, 0], [0, -7], [0, 24], [-46, 0], [0, 8], [-44, 0], [0, -26], [-20, 0], [0, -18], [62, 0], [0, 13], [-34, 0], [52, 0], [-10, 0], [0, -6], [0, -3], [-8, 0], [0, 4], [0, 9], [2, 0], [-92, 0], [0, 12], [0, -5], [-68, 0], [74, 0], [0, -18], [16, 0], [86, 0], [0, -9], [0, 8], [0, 16], [0, -20], [0, -3], [-58, 0], [0, 1], [0, 1], [0, 17], [-56, 0], [62, 0], [0, 9], [-56, 0], [-10, 0], [0, 7], [-20, 0], [0, -19], [0, 6], [-44, 0], [-34, 0], [0, 4], [0, -9], [2, 0], [0, -29], [-10, 0], [0, 11], [0, -24], [-8, 0], [0, 20], [16, 0], [0, 28], [-8, 0], [50, 0], [4, 0], [0, -18], [-22, 0], [0, -11], [0, -23], [100, 0], [0, -19], [86, 0], [88, 0], [0, 3], [52, 0], [0, -16], [62, 0], [0, 4], [-80, 0], [0, 13], [0, 5], [16, 0], [62, 0], [0, 24], [40, 0], [0, -10], [74, 0], [0, -20], [0, 3], [-44, 0], [2, 0], [0, 13], [0, -13], [-92, 0], [0, 27], [0, 2], [0, -16], [-34, 0], [0, -3], [0, 12], [74, 0], [0, 23], [0, 23], [0, 18], [40, 0], [0, 10]]
